/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package application

import (
	"github.com/presslabs/controller-util/syncer"
	settings "k8s.libre.sh/application/settings"
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"
	objects "k8s.libre.sh/objects"
)

type Component interface {
	meta.Instance
	GetMeta() meta.Instance
	SetDefaults()
	Init()
	//	Init(r interfaces.Reconcile, owner interfaces.Object)
	GetObjects() map[int]objects.Object
	GetDependencies() settings.Config
	HasSettings() []string
}

type ComponentReconciler struct {
	ReconcilerBase *ReconcilerBase
	Component      Component
}

func InitComponent(r interfaces.Reconcile, owner interfaces.Object) {

}

func NewObjectSyncersFromComponent(i Component, r interfaces.Reconcile, owner interfaces.Object) (syncers map[int]syncer.Interface) {

	objs := i.GetObjects()
	syncers = make(map[int]syncer.Interface, len(objs))

	for k, obj := range objs {
		s := objects.NewObjectSyncer(obj, owner, r)
		syncers[k] = s
	}

	return syncers
}

func SetComponentSettings(i Component, s map[string]settings.Component) {
	for _, dep := range i.HasSettings() {
		foo := i.GetDependencies()
		foo = settings.MergeSettings(s[dep].GetConfig(), foo)
	}
}

func InitComponentsFromSettings(cs map[string]Component, setts map[string]settings.Component) {
	for _, c := range cs {
		SetComponentSettings(c, setts)
	}
}
