/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package ingress

import (
	interfaces "k8s.libre.sh/interfaces"
	meta "k8s.libre.sh/meta"

	networkingv1beta1 "k8s.io/api/networking/v1beta1"
)

// SecretRef represents a reference to a Secret
type SecretRef string

// +kubebuilder:object:generate=true
type Ingress struct {
	*meta.ObjectMeta               `json:"meta,omitempty"`
	*networkingv1beta1.IngressSpec `json:",inline"`
}

func (obj *Ingress) Init() {
	if obj.ObjectMeta == nil {
		obj.ObjectMeta = &meta.ObjectMeta{}
	}

	if obj.ObjectMeta.Labels == nil {
		obj.ObjectMeta.Labels = make(map[string]string)
	}
}

func (r *Ingress) GetIngressRules() []networkingv1beta1.IngressRule { return r.Rules }
func (r *Ingress) GetIngressTLS() []networkingv1beta1.IngressTLS    { return r.TLS }
func (r *Ingress) GetIngressClassName() *string                     { return r.IngressClassName }

func (o *Ingress) Mutate(obj interfaces.Object) error {
	//	o.Service.Meta = o.Meta
	MutateIngress(o, obj.(*networkingv1beta1.Ingress))
	meta.MutateMeta(o, obj)
	return nil
}

func (o *Ingress) GetObject() interfaces.Object {
	return &networkingv1beta1.Ingress{}
}

func (o *Ingress) GetObjectKind() string  { return "Ingress" }
func (o *Ingress) GetObjectGroup() string { return "network" }
