/*

Licensed under the GNU AFFERO GENERAL PUBLIC LICENSE Version 3 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.gnu.org/licenses/agpl-3.0.html

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package pod

import (
	corev1 "k8s.io/api/core/v1"
	container "k8s.libre.sh/objects/container"
)

type Mutate interface {
	GetPodSecurityContext() *corev1.PodSecurityContext
	GetPodVolumes() []corev1.Volume
	GetPodRestartPolicy() corev1.RestartPolicy
	GetPodNodeSelector() map[string]string
	GetPodAffinity() *corev1.Affinity
	GetPodTolerations() []corev1.Toleration
	GetContainers() []container.Mutate
	container.Mutate
	//	GetInitContainers() []container.Mutate
}

func MutatePod(r Mutate, obj *corev1.PodTemplateSpec) error {

	containerSpec := &corev1.Container{}
	containerList := []corev1.Container{}

	obj.Spec.SecurityContext = r.GetPodSecurityContext()
	obj.Spec.Volumes = r.GetPodVolumes()
	obj.Spec.RestartPolicy = r.GetPodRestartPolicy()
	obj.Spec.NodeSelector = r.GetPodNodeSelector()
	obj.Spec.Affinity = r.GetPodAffinity()
	obj.Spec.Tolerations = r.GetPodTolerations()

	containers := r.GetContainers()
	if len(containers) > 0 {
		for k, c := range containers {
			if len(obj.Spec.Containers) == 0 {
				container.MutateContainer(c, containerSpec)
				containerList = append(containerList, *containerSpec)
				obj.Spec.Containers = containerList
			} else {
				// TODO FIX mutate container by name ?
				container.MutateContainer(c, &obj.Spec.Containers[k])
			}
		}
	}

	// TODO add init containers

	if r.GetPodSecurityContext() != nil {

	}

	return nil
}
